; 

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

align 16

utoa proc src:DWORD, lpdest:DWORD
  ; ----------------------------------------------------------------------
  ; This is a modified version of an algorithm written by Ray Filiatreault
  ; ----------------------------------------------------------------------
     mov eax, [esp+4]            ; src
    mov ecx, [esp+8]            ; lpdest             ; buffer address

    push esi
    push ecx                    ; save dest address

    sub ecx, 1
    mov esi, 0CCCCCCCDh         ; "magic number" multiplier for division by 10
    mov DWORD PTR [esp+16], 10

  @@:
    mul esi                     ; multiply by magic number
    shrd eax, edx, 3            ; binary fractional "remainder" back in EAX
    shr edx, 3                  ; EDX = quotient
    add eax, 1                  ; precaution against occasional "underflow"
    mov [esp+12], edx           ; save current quotient for next division
    mul DWORD PTR [esp+16]      ; x10 gets "decimal" remainder into EDX
    add ecx, 1                  ; increment digit in buffer
    add dl, 30h                 ; convert remainder to ascii
    mov [ecx], dl               ; insert it into buffer
    mov eax, [esp+12]           ; retrieve current quotient
    test eax, eax               ; test if done
    jnz @B                      ; continue if not done

    mov byte ptr [ecx+1], 0     ; terminate the string
    mov esi, [esp]              ; copy the dest address into ESI

  @@:
    movzx eax, BYTE PTR [esi]   ; reverse the string
    movzx edx, BYTE PTR [ecx]
    mov [esi], dl
    mov [ecx], al
    add esi, 1
    sub ecx, 1
    cmp esi, ecx
    jb @B

    pop eax                     ; restore and return dest address
    pop esi

    ret 8

utoa endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef

;  

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

    align 16

utoa_ex proc uvar:DWORD,pbuffer:DWORD

  ; --------------------------------------------------------------------------------
  ; this algorithm was written by Paul Dixon and has been converted to MASM notation
  ; --------------------------------------------------------------------------------

    mov eax, [esp+4]                ; uvar      : unsigned variable to convert
    mov ecx, [esp+8]                ; pbuffer   : pointer to result buffer

    push esi
    push edi

    jmp udword

  align 4
  chartab:
    dd "00","10","20","30","40","50","60","70","80","90"
    dd "01","11","21","31","41","51","61","71","81","91"
    dd "02","12","22","32","42","52","62","72","82","92"
    dd "03","13","23","33","43","53","63","73","83","93"
    dd "04","14","24","34","44","54","64","74","84","94"
    dd "05","15","25","35","45","55","65","75","85","95"
    dd "06","16","26","36","46","56","66","76","86","96"
    dd "07","17","27","37","47","57","67","77","87","97"
    dd "08","18","28","38","48","58","68","78","88","98"
    dd "09","19","29","39","49","59","69","79","89","99"

  udword:
    mov esi, ecx                    ; get pointer to answer
    mov edi, eax                    ; save a copy of the number

    mov edx, 0D1B71759h             ; =2^45\10000    13 bit extra shift
    mul edx                         ; gives 6 high digits in edx

    mov eax, 68DB9h                 ; =2^32\10000+1

    shr edx, 13                     ; correct for multiplier offset used to give better accuracy
    jz short skiphighdigits         ; if zero then don't need to process the top 6 digits

    mov ecx, edx                    ; get a copy of high digits
    imul ecx, 10000                 ; scale up high digits
    sub edi, ecx                    ; subtract high digits from original. EDI now = lower 4 digits

    mul edx                         ; get first 2 digits in edx
    mov ecx, 100                    ; load ready for later

    jnc short next1                 ; if zero, supress them by ignoring
    cmp edx, 9                      ; 1 digit or 2?
    ja  short ZeroSupressed         ; 2 digits, just continue with pairs of digits to the end

    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dh                   ; but only write the 1 we need, supress the leading zero
    inc esi                         ; update pointer by 1
    jmp short ZS1                   ; continue with pairs of digits to the end

  next1:
    mul ecx                         ; get next 2 digits
    jnc short next2                 ; if zero, supress them by ignoring
    cmp edx, 9                      ; 1 digit or 2?
    ja  short ZS1a                  ; 2 digits, just continue with pairs of digits to the end

    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dh                   ; but only write the 1 we need, supress the leading zero
    inc esi                         ; update pointer by 1
    jmp short ZS2                   ; continue with pairs of digits to the end

  next2:
    mul ecx                         ; get next 2 digits
    jnc short next3                 ; if zero, supress them by ignoring
    cmp edx, 9                      ; 1 digit or 2?
    ja  short ZS2a                  ; 2 digits, just continue with pairs of digits to the end

    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dh                   ; but only write the 1 we need, supress the leading zero
    inc esi                         ; update pointer by 1
    jmp short ZS3                   ; continue with pairs of digits to the end

  next3:

  skiphighdigits:
    mov eax, edi                    ; get lower 4 digits

    mov ecx, 100

    mov edx, 28F5C29h               ; 2^32\100 +1
    mul edx
    jnc short next4                 ; if zero, supress them by ignoring
    cmp edx, 9                      ; 1 digit or 2?
    ja  short ZS3a                  ; 2 digits, just continue with pairs of digits to the end

    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dh                   ; but only write the 1 we need, supress the leading zero
    inc esi                         ; update pointer by 1
    jmp short  ZS4                  ; continue with pairs of digits to the end

  next4:
    mul ecx                         ; this is the last pair so don; t supress a single zero
    cmp edx, 9                      ; 1 digit or 2?
    ja  short ZS4a                  ; 2 digits, just continue with pairs of digits to the end

    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dh                   ; but only write the 1 we need, supress the leading zero
    mov byte ptr [esi+1], 0         ; zero terminate string

    jmp short  sdwordend            ; all done

  ZeroSupressed:
    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dx
    add esi, 2                      ; write them to answer

  ZS1:
    mul ecx                         ; get next 2 digits
  ZS1a:
    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dx                   ; write them to answer
    add esi, 2

  ZS2:
    mul ecx                         ; get next 2 digits
  ZS2a:
    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dx                   ; write them to answer
    add esi, 2

  ZS3:
    mov eax, edi                    ; get lower 4 digits
    mov edx, 28F5C29h               ; 2^32\100 +1
    mul edx                         ; edx= top pair
  ZS3a:
    mov edx, chartab[edx*4]         ; look up 2 digits
    mov [esi], dx                   ; write to answer
    add esi, 2                      ; update pointer

  ZS4:
    mul ecx                         ; get final 2 digits
  ZS4a:
    mov edx, chartab[edx*4]         ; look them up
    mov [esi], dx                   ; write to answer

    mov byte ptr [esi+2], 0         ; zero terminate string

  sdwordend:

    pop edi
    pop esi

    ret 8

utoa_ex endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef

;  

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

ALIGN 16
; u32toa by brethren
 db  11 Dup (0)
utoa2 PROC, value:DWORD, szbuf:DWORD
	pop     edx	        	
	pop     eax           
	cmp    eax, 99999              
	pop     ecx        
	jbe      L4	
	cmp     eax, 9999999        
	push    edx				
	jbe     L2
	cmp    eax, 999999999    
        push   ebx 	
	ja       L8
	cmp    eax, 99999999     
	ja       L1	
	add    ecx, 8					 	
	push   esi    
	jne     L9
L1 :
	add    ecx, 9
	push   esi							
	jne     L9
L2:
	cmp    eax, 999999         
	push   ebx
	ja       L3	
	add    ecx, 6
	push   esi							
	jne     L9
L3:
	add    ecx, 7
	push   esi							
	jne     L9
L4:	
	cmp    eax, 999              
	push   edx        
	jbe     L6
	cmp    eax, 9999              	
	push   ebx
	ja       L5
	add    ecx, 4
	push   esi							
	jne     L9
L5:
	add    ecx, 5
	push   esi							
	jne     L9
L6:
	cmp    eax, 99			            
	push   ebx
	jbe     L7
	add    ecx, 3
	push   esi							
	jne     L9
L8:
	push   esi    
	add    ecx, 10
L9:
	mov    byte ptr [ecx], 0
	mov    esi,  0CCCCCCCDh
@@:
    mov	ebx, eax
    mul	esi
    shr	edx, 3
    sub	ecx, 1
    mov	eax, edx
    lea	edx, [edx+edx*4-18h]
    lea	edx, [edx+edx]
    sub	ebx, edx
    mov	[ecx], bl
    test eax, eax
    je   @f

    mov	ebx, eax
    mul	esi
    shr	edx, 3
    sub	ecx, 1
    mov	eax, edx
    lea	edx, [edx+edx*4-18h]
    lea	edx, [edx+edx]
    sub	ebx, edx
    mov	[ecx], bl
    test eax, eax
    jne  @b
@@:
    pop	esi	
    mov	eax, ecx
    pop	ebx 
    ret 	
L7:
    cmp     eax, 9			           
    push    esi
    jbe     L10
    add     ecx, 2                     
    jne     L9
L10:
    add eax, 30h              
    pop	esi	
    mov	[ecx],  ax
    pop	ebx 
    mov	eax, ecx
    ret 

    push ebx
    push esi

    mov eax, [esp+4][8]       ; value
    mov esi, [esp+8][8]       ; szbuf

    cmp eax, 9999
    ja lb4

    cmp eax, 999999
    ja lb6

    cmp eax, 9
    ja lb1
    add esi, 1
    jmp nxt
  lb1:
    cmp eax, 99
    ja lb2
    add esi, 2
    jmp nxt
  lb2:
    cmp eax, 999
    ja lb3
    add esi, 3
    jmp nxt
  lb3:
    cmp eax, 9999
    ja lb4
    add esi, 4
    jmp nxt
  lb4:
    cmp eax, 99999
    ja lb5
    add esi, 5
    jmp nxt
  lb5:
    cmp eax, 999999
    ja lb6
    add esi, 6
    jmp nxt
  lb6:
    cmp eax, 9999999
    ja lb7
    add esi, 7
    jmp nxt
  lb7:
    cmp eax, 99999999
    ja lb8
    add esi, 8
    jmp nxt
  lb8:
    cmp eax, 999999999
    ja lb9
    add esi, 9
    jmp nxt
  lb9:
    add esi, 10

  nxt:
    mov ebx, 0CCCCCCCDh
    mov BYTE PTR [esi], 0
    sub esi, 1

  stlp:
    mov ecx, eax
    mul ebx
    shr edx, 3
    mov eax, edx
    lea edx, [edx+edx*4]
    lea edx, [edx+edx-48]
    sub ecx, edx
    mov [esi], cl
    sub esi, 1
    test eax, eax
    jbe lpqt

    mov ecx, eax
    mul ebx
    shr edx, 3
    mov eax, edx
    lea edx, [edx+edx*4]
    lea edx, [edx+edx-48]
    sub ecx, edx
    mov [esi], cl
    sub esi, 1
    test eax, eax
    ja stlp

  lpqt:
    pop esi
    pop ebx

    ret 8

utoa2 ENDP

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef

;  

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE

    align 16
utoa3 proc dwNumber:DWORD, pszString:DWORD
    mov edx, [esp+4]

    cmp edx, 9
    ja @F
    mov edx, 1
    jmp nxt

  @@:
    cmp edx, 99
    ja @F
    mov edx, 2
    jmp nxt

  @@:
    cmp edx, 999
    ja @F
    mov edx, 3
    jmp nxt

  @@:
    cmp edx, 9999
    ja @F
    mov edx, 4
    jmp nxt

  @@:
    cmp edx, 99999
    ja @F
    mov edx, 5
    jmp nxt

  @@:
    cmp edx, 999999
    ja @F
    mov edx, 6
    jmp nxt

  @@:
    cmp edx, 9999999
    ja @F
    mov edx, 7
    jmp nxt

  @@:
    cmp edx, 99999999
    ja @F
    mov edx, 8
    jmp nxt

  @@:
    cmp edx, 999999999
    ja @F
    mov edx, 9
    jmp nxt

  @@:
    mov edx, 10

  nxt:
    mov eax, [esp+4]            ; src
    mov ecx, [esp+8]            ; lpdest             ; buffer address

    push ebx
    push esi
    push edi

    mov esi, 0CCCCCCCDh         ; "magic number" multiplier for division by 10
    mov ebx, 10
    add ecx, edx                ; add required buffer length to start address
    mov BYTE PTR [ecx], 0       ; write the terminator
    sub ecx, 1                  ; decrement down to first write position

  @@:
    mul esi                     ; multiply by magic number
    shrd eax, edx, 3            ; binary fractional "remainder" back in EAX
    shr edx, 3                  ; EDX = quotient
    add eax, 1                  ; precaution against occasional "underflow"
    mov edi, edx                ; save current quotient for next division
    mul ebx                     ; x10 gets "decimal" remainder into EDX
    add dl, 30h                 ; convert remainder to ascii
    mov [ecx], dl               ; insert it into buffer
    sub ecx, 1                  ; decrement buffer location
    mov eax, edi                ; retrieve current quotient
    test eax, eax               ; test if done
    jz @F                       ; continue if not done

    mul esi                     ; multiply by magic number
    shrd eax, edx, 3            ; binary fractional "remainder" back in EAX
    shr edx, 3                  ; EDX = quotient
    add eax, 1                  ; precaution against occasional "underflow"
    mov edi, edx                ; save current quotient for next division
    mul ebx                     ; x10 gets "decimal" remainder into EDX
    add dl, 30h                 ; convert remainder to ascii
    mov [ecx], dl               ; insert it into buffer
    sub ecx, 1                  ; decrement buffer location
    mov eax, edi                ; retrieve current quotient
    test eax, eax               ; test if done
    jnz @B                      ; continue if not done


  @@:
    pop edi
    pop esi
    pop ebx

    mov eax, [esp+8]

    ret 8

utoa3 endp

align 16
utoa_exLingo proc src:DWORD, lpdest:DWORD
    mov	   eax,	[esp+1*4]		     
	mov	   edx,	0D1B71759h	     
 	mov	   ecx,	[esp+2*4]		     
	mov	  [esp+1*4], edi		    	
	mov	  edi,	eax						
	mov   [esp+2*4], esi		    	
 	mul	  edx								
	shr	  edx,	13						
	mov	  esi,	100	
	je		  LoNext3				 	    
	imul	  eax,	edx, 10000	
	sub	  edi,	eax
	mov	  eax,	68DB9h		    	
	mul	  edx		
	jnc	  LoNext1	
	cmp	  edx,	9	
	jbe     Lo0
	movzx	edx,	word ptr chartabL[edx+edx]	 
	add    ecx, 8      
	mov   [ecx-8], edx
Lo1:				
	mul	esi	 
Lo1a:			
	movzx	edx,	word ptr chartabL[edx+edx]	 
	mov	[ecx-6], edx	 
Lo2:				
	mul	esi		 
Lo2a:				
	movzx	edx,	word ptr chartabL[edx+edx]	 
	mov	[ecx-4], edx	 
Lo3:				
	mov	eax,	28F5C29h	 
	mul	edi		 
Lo3a:				
	movzx	edx,	word ptr chartabL[edx+edx]	 
	mov	[ecx-2], edx	 
Lo4:				
	mul	esi		 
Lo4a:				
	movzx	eax,	 word ptr chartabL[edx+edx]	 
	pop	 edx		
	mov	[ecx], 	eax	 
	pop	edi	
	lea	eax,	[ecx+2]	
	pop	esi		
	jmp	edx
Lo0:										
    add	ecx,	7
	add	edx,	30h	
	mov	[ecx-7],  edx	
	jne	Lo1	
LoNext1:			
	mul	esi		 
	jnc	@f		 
	add	ecx, 6	
	cmp	edx,	9	 
	ja		Lo1a		 
	add	edx,	30h	
	sub	ecx,	1	 
	mov	[ecx-5],  edx	 
	jnz	Lo2		 
@@:				
	mul	esi		 
	jnc	LoNext3		 
	add	ecx, 	4	
	cmp	edx,	9	
	ja		Lo2a		 
	add	edx,	30h	
	sub	ecx,	1	 
	mov	[ecx-3],  edx	 
	jnz	Lo3		 
LoNext3:			
	mov	eax,	28F5C29h 
	mul	edi		 
	jnc	@f		 
	add	ecx, 	2	
	cmp	edx,	9	 
	ja		Lo3a		 
	add	edx,	30h	
	sub	ecx,	1	 
	mov	[ecx-1],   edx	 
	jnz	Lo4		 
@@:				
	mul	esi		 
	cmp	edx,	9	 
	ja		Lo4a	 
	lea	eax, [edx+30h]
	pop	edx
	mov	[ecx], 	eax	
	pop	edi
	lea	eax,	[ecx+1]	
	pop	esi
	jmp	edx

utoa_exLingo endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef

;  
